declare module "vrame" {
  import type { CSSProperties, Ref, Plugin, Component } from "vue";

  export interface VrameConfig {
    components?: Plugin[];
  }

  export function createVrame(vrameConfig?: VrameConfig): Plugin;

  export const components: Plugin[];

  export interface IIComponentSpinner {
    onSpin: () => void;
    offSpin: () => void;
    isOpen: boolean;
    /** 스피너 세팅 함수, 사용할 때 스피너의 ref가 created에선 아직 할당되지 않았으므로
     * () => spinner.value.setSpinner(anyFunction)() 와 같이 함수로 한 번 감싸주는 형태로 사용해야 한다.
     * @param {F extends (...args: any[]) => any} fn 세팅시 실행해 줄 함수
     * @param {string} style 스피너 배경 스타일 opacity: 불투명, translucent: 반투명
     * @param {Ref<boolean>} status disabled와 같은 상태 ref
     * @returns {any} fn의 리턴값
     */
    setSpinner: <F extends (...args: any[]) => any>(
      fn: F,
      style: "opacity" | "translucent" = "opacity",
      status?: Ref<boolean>
    ) => F;
  }

  // SList
  export interface TSList {
    listStyle?: CSSProperties;
    icon?: { icon: string; color: string };
    title?: string;
    hoverColor?: string;
    hoverBgColor?: string;
    link?: boolean;
    to?: string;
    children?: TSList[];
  }

  // STable
  export interface ISTableStyles {
    tableStyle?: CSSProperties;
    tHeaderStyle?: CSSProperties;
    tBodyStyle?: CSSProperties;
    thStyle?: (idx: number) => CSSProperties;
    trStyle?: (idx: number) => CSSProperties;
    columnStyle?: (idx: number) => CSSProperties;
    tdStyle?: (ridx: number, cidx: number) => CSSProperties;
  }

  export interface TValue {
    text: string;
    value: any;
  }
}

/// <reference types="vite/client" />
import { Ref, UnwrapRef } from "vue";

declare global {
  // ref
  function ref<T extends object>(
    value: T
  ): [T] extends [Ref] ? T : Ref<UnwrapRef<T>>;
  function ref<T>(value: T): Ref<UnwrapRef<T>>;
  function ref<T = any>(): Ref<T | undefined>;

  // reactive
  function reactive<T extends object>(target: T): UnwrapNestedRefs<T>;

  // computed
  function computed<T>(
    getter: ComputedGetter<T>,
    debugOptions?: DebuggerOptions
  ): ComputedRef<T>;

  function computed<T>(
    options: WritableComputedOptions<T>,
    debugOptions?: DebuggerOptions
  ): WritableComputedRef<T>;

  // watch
  export declare function watch<
    T extends MultiWatchSources,
    Immediate extends Readonly<boolean> = false
  >(
    sources: [...T],
    cb: WatchCallback<MapSources<T, false>, MapSources<T, Immediate>>,
    options?: WatchOptions<Immediate>
  ): WatchStopHandle;

  export declare function watch<
    T extends Readonly<MultiWatchSources>,
    Immediate extends Readonly<boolean> = false
  >(
    source: T,
    cb: WatchCallback<MapSources<T, false>, MapSources<T, Immediate>>,
    options?: WatchOptions<Immediate>
  ): WatchStopHandle;

  export declare function watch<T, Immediate extends Readonly<boolean> = false>(
    source: WatchSource<T>,
    cb: WatchCallback<T, Immediate extends true ? T | undefined : T>,
    options?: WatchOptions<Immediate>
  ): WatchStopHandle;

  export declare function watch<
    T extends object,
    Immediate extends Readonly<boolean> = false
  >(
    source: T,
    cb: WatchCallback<T, Immediate extends true ? T | undefined : T>,
    options?: WatchOptions<Immediate>
  ): WatchStopHandle;
}

export {};
